
package drawables;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.ListIterator;
import javax.swing.DefaultBoundedRangeModel;

/**
 *
 * @author development
 */
public class CachingLayer extends AbstractLayer {

    BufferedImage cacheImage = null;
    String name = "default";

    public CachingLayer( ) {
        this("default");
    }
    public CachingLayer(String s) {
        name = s;
    }

    @Override
    public void invalidateCache() {
        cacheImage = null;
    }


    @Override
    public boolean valueChanged(DefaultBoundedRangeModel m, boolean change) {
        if (change) {
            this.invalidateCache();
        }
        return change;
    }

    @Override
    public boolean hasValidCachedImage() {
        return cacheImage != null;
    }

    /**
     *
     * @param paintTool
     * @param copyTool
     * @param lowerLayers
     * @param area
     * @param referenceFrame
     */
    @Override
    protected void renderWith(Graphics2D paintTool,
        Graphics2D copyTool,
        ListIterator<AbstractLayer> lowerLayers,
        Rectangle area, ReferenceFrame referenceFrame) {
        if (!this.hasValidCachedImage()) {
            //System.out.println("repaint cache");
            cacheImage = new BufferedImage(area.width, area.height, BufferedImage.TYPE_INT_ARGB);
            Graphics2D gc = cacheImage.createGraphics();
            Graphics2D localCopyTool = cacheImage.createGraphics();

            int width = area.width;
            int height = area.height;

            double scaleFactor = Math.min(height /(double)referenceFrame.height(),
                    width /(double)referenceFrame.width());
            gc.translate(width/2.0d, height/2.0d);
            gc.scale(scaleFactor, scaleFactor);
            RenderingHints einstellungen = new RenderingHints(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
            gc.addRenderingHints(einstellungen);
            super.renderWith(gc, localCopyTool, lowerLayers, area, referenceFrame);
            gc.dispose();
        } else {

        }
        copyTool.drawImage(cacheImage, null, 0, 0);
    }
    /**
     * to be modified
     * @param drawTool
     * @param copyTool
     * @param destCanvas
     * @param area
     */
    /*
    public void renderWith(Graphics2D drawTool, Graphics2D copyTool,
       BufferedImage destCanvas, Rectangle area) {
        if (cacheImage != null) {
            copyTool.drawImage(cacheImage, null, 0, 0);
        } else {
            //draw here
            cacheImage = new BufferedImage
                    (destCanvas.getWidth(), destCanvas.getHeight(),
                     destCanvas.getType());
            Graphics2D gc = cacheImage.createGraphics();
            gc.drawImage(destCanvas, null, 0, 0);
            gc.dispose();
        }
    }
  */
}
