
package drawables;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Path2D;
import java.util.ListIterator;


/**
 *
 * @author Projekte
 */
public class EllipticScaleTicksLayer extends AbstractEllipticalVisualElement
{

    private ScaleProperties scaleProperties;
    private TickSequenceRule tickSequenceRule;
    private double radius2;

    public EllipticScaleTicksLayer (
            AngularRange ar,
            double r,
            double r2,
            ScaleProperties sp,
            TickSequenceRule tsr,
            VisualProperties vp) {
        super(ar, r, vp);
        scaleProperties = sp;
        tickSequenceRule = tsr;
        radius2 = r2;
    }

    @Override protected void renderWith (Graphics2D paintTool,
            Graphics2D copyTool,
            ListIterator<AbstractLayer> iterator,
            Rectangle area,
            ReferenceFrame rf) {
        super.renderWith(paintTool, copyTool, iterator, area, rf);
        double startAngle = angularRange.startAngle;
        double endAngle = angularRange.endAngle();
        int nroOfSections = scaleProperties.numberOfSections;
        double delta = (endAngle - startAngle)/nroOfSections;
        visualProperties.installInto(paintTool);
        AffineTransform transformation = paintTool.getTransform();
        paintTool.translate(angularRange.center.getX(), angularRange.center.getY());
        AffineTransform t2 = paintTool.getTransform();
        Path2D.Double tickline = new Path2D.Double();
        tickline.moveTo(radius, 0.0);
        tickline.lineTo(radius2, 0.0);
          //  It is sufficient to create this path once. It can be used
          //  to draw all tick lines.

        TickSequence s = new TickSequence(scaleProperties, tickSequenceRule);

        while (s.hasMoreElements()) {
            int tick = s.nextElement();
              //  rotate the coordinate system before the needle is drawn:
            paintTool.rotate(Math.toRadians(startAngle + delta*tick));
            paintTool.draw(tickline);
            paintTool.setTransform(t2);  //  restore coordinate system
        }
        paintTool.setTransform(transformation);
           //  drop all transformations made in this method and
           //  restore the transformation that was active when the
           //  method was called.
    }
}
