
package drawables;

import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Path2D;
import java.awt.geom.Point2D;
import java.util.ListIterator;

/**
 *
 * @author Projekte
 */
public class FlatScaleTicksLayer extends AbstractFlatVisualElement {
    ScaleProperties scaleProperties;
    TickSequenceRule tickSequenceRule;
    double upperStop, lowerStop;

    public FlatScaleTicksLayer (StraightRange r,
            ScaleProperties sp,
            double s1,
            double s2,
            TickSequenceRule tsr,
            VisualProperties vp) {
        super(r, vp);
        upperStop = s1;
        lowerStop = s2;
        scaleProperties = sp;
        tickSequenceRule = tsr;
    }

    @Override protected void renderWith (Graphics2D paintTool,
            Graphics2D copyTool,
            ListIterator<AbstractLayer> iterator,
            Rectangle area,
            ReferenceFrame rf) {
        super.renderWith(paintTool, copyTool, iterator, area, rf);
        visualProperties.installInto(paintTool);
        AffineTransform transformation = paintTool.getTransform();

        if (range.isHorizontal()) {
            this.renderHorizontalScaleWith(paintTool);
        } else {
            this.renderVerticalScaleWith(paintTool);
        }
        paintTool.setTransform(transformation);
    }

    private void renderHorizontalScaleWith(Graphics2D paintTool) {
        Point2D startPnt = range.startPoint;
        double startPos = startPnt.getX();
        double delta = (double)range.length/scaleProperties.numberOfSections;
        paintTool.translate(0.0, startPnt.getY());
           //  movethe origin of the coordinate to the center point
           //  of the scale.


        AffineTransform t2 = paintTool.getTransform();
        Path2D.Double tickline = new Path2D.Double();
        tickline.moveTo(0.0, upperStop);
        tickline.lineTo(0.0, lowerStop);
          //  It is sufficient to create this path once. It can be used
          //  to draw all tick lines.

        TickSequence s = new TickSequence(scaleProperties, tickSequenceRule);

        while (s.hasMoreElements()) {
            int tick = s.nextElement();
            paintTool.translate(startPos + tick*delta, 0.0);
            paintTool.draw(tickline);
            paintTool.setTransform(t2);
        }
    }

    private void renderVerticalScaleWith(Graphics2D paintTool) {
        Point2D startPnt = range.startPoint;
        double startPos = startPnt.getY();
        double delta = (double)range.length/scaleProperties.numberOfSections;
        paintTool.translate(startPnt.getX(), 0.0);
           //  movethe origin of the coordinate to the center point
           //  of the scale.
        
        AffineTransform t2 = paintTool.getTransform();
        Path2D.Double tickline = new Path2D.Double();
        tickline.moveTo(upperStop, 0.0);
        tickline.lineTo(lowerStop, 0.0);
          //  It is sufficient to create this path once. It can be used
          //  to draw all tick lines.

        TickSequence s = new TickSequence(scaleProperties, tickSequenceRule);

        while (s.hasMoreElements()) {
            int tick = s.nextElement();
            paintTool.translate(0.0, startPos + tick*delta);
            paintTool.draw(tickline);
            paintTool.setTransform(t2);
        }
    }
}
