
package drawables;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.LinkedList;
import java.util.ListIterator;

import javax.swing.DefaultBoundedRangeModel;

/**
 *
 * @author Projekte
 */
public class LayeredDisplayObject {
    protected LinkedList<AbstractLayer> layers;
    ReferenceFrame referenceFrame;

    public LayeredDisplayObject (LinkedList<AbstractLayer> lal,
            ReferenceFrame rf) {
        layers = lal;
        referenceFrame = rf;
    }

    /**
     *
     * @param drawTool
     * @param copyTool
     * @param destCanvas
     * @param area
     */
    public void renderOn (Graphics2D drawTool, Graphics2D copyTool,
              BufferedImage destCanvas, Rectangle area){
        int width = area.width;
        int height = area.height;

        if (layers.isEmpty()) {
            return;
        }

        double scaleFactor = Math.min(height/(double)referenceFrame.height(),
                width/(double)referenceFrame.width());
        drawTool.translate(width/2.0d, height/2.0d);
        drawTool.scale(scaleFactor, scaleFactor);
        RenderingHints einstellungen = new RenderingHints(
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON);
        drawTool.addRenderingHints(einstellungen);
        
        ListIterator<AbstractLayer> iterator = layers.listIterator(layers.size());
        if (iterator.hasPrevious()) {
            AbstractLayer layer = iterator.previous();
            layer.renderWith(drawTool, copyTool, iterator, area, referenceFrame);
        }
    }

    /**
     * Drop all cached images. This is a necessary operation when the presentation
     * window of the instrument is resized.
     */
    public void invalidateCaches() {
        ListIterator<AbstractLayer> iterator = layers.listIterator();
        while (iterator.hasNext()) {
            iterator.next().invalidateCache();
        }
    }

    /**
     * The top-most layer is the last one in the list, the bottom-most
     * is at layers[0].
     * @param m
     */
    public void invalidateCaches(DefaultBoundedRangeModel m) {
        ListIterator<AbstractLayer> iterator = layers.listIterator();
        boolean changeNoticed = false;
        while (iterator.hasNext()) {
            changeNoticed = iterator.next().valueChanged(m, changeNoticed);

        }
    }
}
