
package drawables;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.util.ListIterator;

/**
 *
 * @author Projekte
 */
public class TickLabelsOnEllipticScale extends AbstractEllipticalVisualElement {

    private ScaleProperties scaleProperties;
    private TickSequenceRule tickSequenceRule;
    private VisualStringProperties visualStringProperties;

    public TickLabelsOnEllipticScale(
            AngularRange ar,
            double r,
            TickSequenceRule tsr,
            VisualProperties vp,
            VisualStringProperties vsp,
            ScaleProperties sp) {
        super(ar, r, vp);
        scaleProperties = sp;
        tickSequenceRule = tsr;
        visualStringProperties = vsp;
    }

    @Override protected void renderWith (Graphics2D paintTool,
            Graphics2D copyTool,
            ListIterator<AbstractLayer> iterator,
            Rectangle area, ReferenceFrame rf) {
        super.renderWith(paintTool, copyTool, iterator, area, rf);
        double startAngle = angularRange.startAngle;
        double endAngle = angularRange.endAngle();
        int nroOfSections = scaleProperties.numberOfSections;
        double delta = (endAngle - startAngle)/nroOfSections;

        AffineTransform transformation = paintTool.getTransform();
        paintTool.translate(angularRange.center.getX(), angularRange.center.getY());
           //  move the origin of the coordinate to the center point
           //  of the scale.

        Font schrift = visualStringProperties.font;
        FontMetrics metrics = paintTool.getFontMetrics(schrift);
        String text;
        AffineTransform t2 = paintTool.getTransform();
        paintTool.setFont(schrift);
        paintTool.setColor(visualStringProperties.textColor);

        TickSequence s = new TickSequence(scaleProperties, tickSequenceRule);

        while (s.hasMoreElements()) {
            int tick = s.nextElement();
            int labelValue = scaleProperties.startValue +
                    (int)(scaleProperties.delta()*tick);
            text = Integer.toString(labelValue);
            float textbreite = metrics.stringWidth(text);
            float xPos = - textbreite / 2;
            float yHeight = metrics.getMaxAscent();

            paintTool.rotate(Math.toRadians(startAngle + tick*delta));
            paintTool.translate(radius - yHeight, 0);
            paintTool.rotate(Math.toRadians(90.0));
            paintTool.drawString(text, xPos, 0);
            paintTool.setTransform(t2);
        }
        paintTool.setTransform(transformation);
    }
}
