
package drawables;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.util.ListIterator;

/**
 *
 * @author Projekte
 */
public class TickLabelsOnFlatScale extends AbstractFlatVisualElement {

    //  eine Alignment-Angabe wird hier noch benötigt
    protected double labelPos;
    ScaleProperties scaleProperties;
    TickSequenceRule tickSequenceRule;
    VisualStringProperties visualStringProperties;

    public TickLabelsOnFlatScale (StraightRange r,
            double pos,
            TickSequenceRule tsr,
            VisualProperties vp,
            VisualStringProperties vsp,
            ScaleProperties sp) {
        super(r, vp);
        labelPos = pos;
        scaleProperties = sp;
        tickSequenceRule = tsr;
        visualStringProperties = vsp;
    }

    @Override protected void renderWith (Graphics2D paintTool,
            Graphics2D copyTool,
            ListIterator<AbstractLayer> iterator,
            Rectangle area,
            ReferenceFrame rf) {
        super.renderWith(paintTool, copyTool, iterator, area, rf);
        AffineTransform transformation = paintTool.getTransform();
        if (range.isHorizontal()) {
            this.renderHorizontalWith(paintTool);
        } else {
            this.renderVerticalWith(paintTool);
        }
        paintTool.setTransform(transformation);
    }

    private void renderHorizontalWith(Graphics2D paintTool) {
        Point2D startPnt = range.startPoint;
        double startPos = startPnt.getX();
        double delta = (double)range.length/scaleProperties.numberOfSections;

        paintTool.translate(0.0, startPnt.getY());
           //  move the origin of the coordinate to the center point
           //  of the scale.

        Font schrift = visualStringProperties.font;
        FontMetrics metrics = paintTool.getFontMetrics(schrift);
        String text;
        AffineTransform t2 = paintTool.getTransform();
        paintTool.setFont(schrift);
        paintTool.setColor(visualStringProperties.textColor);

        TickSequence s = new TickSequence(scaleProperties, tickSequenceRule);

        while (s.hasMoreElements()) {
            int tick = s.nextElement();
            int labelValue = scaleProperties.startValue +
                    (int)(scaleProperties.delta()*tick);
            text = Integer.toString(labelValue);
            float textbreite = metrics.stringWidth(text);
            float xPos = - textbreite / 2;
            float yHeight = metrics.getMaxAscent() - metrics.getMaxDescent();
            paintTool.translate(startPos + tick*delta, labelPos);
            paintTool.drawString(text, xPos, yHeight / 2);
            paintTool.setTransform(t2);
        }
    }

    private void renderVerticalWith(Graphics2D paintTool) {
        Point2D startPnt = range.startPoint;
        double startPos = startPnt.getY() + range.length;
        double delta = (double)range.length/scaleProperties.numberOfSections;

        paintTool.translate(startPnt.getX(), 0.0);
           //  move the origin of the coordinate to the center point
           //  of the scale.

        Font schrift = visualStringProperties.font;
        FontMetrics metrics = paintTool.getFontMetrics(schrift);
        String text;
        AffineTransform t2 = paintTool.getTransform();
        paintTool.setFont(schrift);
        paintTool.setColor(visualStringProperties.textColor);

        TickSequence s = new TickSequence(scaleProperties, tickSequenceRule);

        while (s.hasMoreElements()) {
            int tick = s.nextElement();
            int labelValue = scaleProperties.startValue +
                    (int)(scaleProperties.delta()*tick);
            text = Integer.toString(labelValue);
            float textbreite = metrics.stringWidth(text);
            float xPos = - textbreite / 2;
            float yHeight = metrics.getMaxAscent() - metrics.getMaxDescent();
            paintTool.translate(labelPos, startPos - tick*delta);
            paintTool.drawString(text, xPos, yHeight / 2);
            paintTool.setTransform(t2);
        }
    }
}
