
package example001;
import common.SliderListener;
import drawables.AbstractLayer;
import drawables.AngularRange;
import drawables.BackgroundLayer;
import drawables.CachingLayer;
import drawables.CircularArcLayer;
import drawables.EllipticScaleTicksLayer;
import drawables.LayeredDisplayObject;
import drawables.Needle;
import drawables.NeedleLayer;
import drawables.ReferenceFrame;
import drawables.ScaleProperties;
import drawables.SimpleNeedle;
import drawables.TickLabelsOnEllipticScale;
import drawables.TickSequenceRule;
import drawables.VisualProperties;
import drawables.VisualStringProperties;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.util.Calendar;
import java.util.LinkedList;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.Timer;
import mvc.ComponentContainer;
import window.InstrumentExplorerModel;
import window.SimpleInstrument;
import window.SimpleInstrumentModel;

/**
 * Eine Uhr mit Sekundenzeiger.
 * @author Projekte
 */
public class ClockExample001 extends SimpleInstrumentModel
                             implements ActionListener {

    private LayeredDisplayObject instrument;
    private DefaultBoundedRangeModel hourValues;
    private DefaultBoundedRangeModel minuteValues;
    private DefaultBoundedRangeModel secondValues;
    private SliderListener [] valueListeners;
    private Timer timer;

    public ClockExample001() {
        this.createValueSource();
        this.createInstrument();
    }


    @Override
    public LayeredDisplayObject getInstrument() {
        return instrument;
    }

    @Override
    public SliderListener [] getValueListeners() {
         return valueListeners;
     }

    @Override
    public void installNewComponent(ComponentContainer<InstrumentExplorerModel> cc) {
        valueListeners = new SliderListener[3];
        valueListeners[0] = new SliderListener(hourValues);
        valueListeners[1] = new SliderListener(minuteValues);
        valueListeners[2] = new SliderListener(secondValues);
        Calendar currentDate = Calendar.getInstance();
        hourValues.setValue(currentDate.get(Calendar.HOUR)*10 + currentDate.get(Calendar.MINUTE)/6);
        minuteValues.setValue(currentDate.get(Calendar.MINUTE));
        secondValues.setValue(currentDate.get(Calendar.SECOND));
        SimpleInstrument view = new SimpleInstrument(this);
        view.installInto(cc);
        timer = new Timer(500, this);
        timer.start();
    }

    private void createValueSource() {
        hourValues = new DefaultBoundedRangeModel(0, 0, 0, 120);
        minuteValues = new DefaultBoundedRangeModel(0, 0, 0, 60);
        secondValues = new DefaultBoundedRangeModel(0, 0, 0, 60);

    }

    private void createInstrument() {
        LinkedList<AbstractLayer> layers;
        ReferenceFrame frame = new ReferenceFrame(-100, 100, -100, 100);
        Point2D cp = new Point2D.Double(0.0d, 0.0d);
        AngularRange a = new AngularRange(cp, -90.0d, 360.0d);
        ScaleProperties sp = new ScaleProperties(0, 59, 60);   // Minutenskala
        ScaleProperties sp2 = new ScaleProperties(0, 12, 12);  // Stundenskala
        VisualProperties vp = new VisualProperties(Color.BLACK, 1.0f);
        VisualStringProperties vsp = new VisualStringProperties(Color.BLACK, new Font ("Courier", Font.PLAIN, 10));
        Needle needleHours =
                new Needle(a, new ScaleProperties(0, 119, 120),
                          new SimpleNeedle(61.0, 1.0, 5.0, 4.2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT),
                          new Color(0, 0, 0));
        Needle needleMinutes =
                new Needle(a, sp,
                          new SimpleNeedle(70.0, 1.0, 5.0, 4.2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT),
                          new Color(0, 0, 0));

        Needle needleSeconds =
                new Needle(a, sp,
                          new SimpleNeedle(82.0, 1.0, 5.0, 4.2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT),
                          new Color(255, 0, 0));

        layers = new LinkedList<>();
        layers.addLast(new BackgroundLayer(Color.WHITE, frame));
        layers.addLast(new EllipticScaleTicksLayer(a, 95.0, 90.0, sp,
                                                   new TickSequenceRule(1, 1, 5),
                                                   vp));
        layers.addLast(new EllipticScaleTicksLayer(a, 95.0, 80.0, sp,
                                                   new TickSequenceRule(0, 5),
                                                   vp));

        layers.addLast(new CircularArcLayer(a, 95.0, vp));
        layers.addLast(new TickLabelsOnEllipticScale(
                 a, 78.0, new TickSequenceRule(1, 1),
                vp, vsp, sp2));
        layers.addLast(new CachingLayer("zifferblatt"));
        layers.addLast(new NeedleLayer(needleHours, this.hourValues));
        layers.addLast(new NeedleLayer(needleMinutes, this.minuteValues));
        layers.addLast(new CachingLayer("schwarze zeiger"));
        layers.addLast(new NeedleLayer(needleSeconds, this.secondValues));

        instrument = new LayeredDisplayObject(layers, frame);
    }

    @Override
    public void stopInstrument() {
        if (timer != null) {
            timer.stop();
            timer = null;
        }

        valueListeners = null;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        Calendar currentDate = Calendar.getInstance();
        SliderListener listenerToNotify;


        if (secondValues.getValue() == currentDate.get(Calendar.SECOND)) {
            return;
        }
        secondValues.setValue(currentDate.get(Calendar.SECOND));
        try {
            listenerToNotify = valueListeners[2];
            if (currentDate.get(Calendar.MINUTE) != minuteValues.getValue()) {
                minuteValues.setValue(currentDate.get(Calendar.MINUTE));
                hourValues.setValue(currentDate.get(Calendar.HOUR)*10 + currentDate.get(Calendar.MINUTE)/6);
                listenerToNotify = valueListeners[0];
            }
            listenerToNotify.signalChange();
        }
        catch (NullPointerException ex) {

        }
    }
}
