
package example001;

import drawables.AbstractLayer;
import drawables.AngularRange;
import drawables.BackgroundLayer;
import drawables.CachingLayer;
import drawables.CircularArcLayer;
import drawables.EllipticScaleTicksLayer;
import drawables.GaugeLabel;
import drawables.LayeredDisplayObject;
import drawables.Needle;
import drawables.NeedleLayer;
import drawables.ReferenceFrame;
import drawables.ScaleProperties;
import drawables.SimpleNeedle;
import drawables.TickLabelsOnEllipticScale;
import drawables.TickSequenceRule;
import drawables.VisualProperties;
import drawables.VisualStringProperties;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.util.LinkedList;
import javax.swing.DefaultBoundedRangeModel;
import mvc.ComponentContainer;
import window.InstrumentExplorerModel;
import window.TestableInstrument;
import window.TestableInstrumentModel;

/**
 * Tachometer, Instrument verwendet einen Bogen von 270°.
 * @author Projekte
 */
public class InstrumentExample001 extends TestableInstrumentModel {

    private DefaultBoundedRangeModel valueSource;
    private LayeredDisplayObject instrument;

    public InstrumentExample001() {
        this.createValueSource();
        this.createInstrument();
    }

    @Override
    public DefaultBoundedRangeModel getSliderModel() {
        return valueSource;
    }

    @Override
    public LayeredDisplayObject getInstrument() {
        return instrument;
    }

    @Override
    public void installNewComponent(ComponentContainer<InstrumentExplorerModel> cc) {
        TestableInstrument view = new TestableInstrument(this);
        view.installInto(cc);
    }

    private void createValueSource() {
        valueSource = new DefaultBoundedRangeModel(0, 0, 0, 150*5);
    }
    private void createInstrument() {
        LinkedList<AbstractLayer> layers;
        ReferenceFrame frame = new ReferenceFrame(-100, 100, -100, 100);
        Point2D cp = new Point2D.Double(0.0d, 0.0d);
        AngularRange a = new AngularRange(cp, 135.0d, 270.0d);
        ScaleProperties sp = new ScaleProperties(0, 150, 150);
        VisualProperties vp = new VisualProperties(Color.WHITE, 1.0f);
        VisualStringProperties vsp = new VisualStringProperties(Color.WHITE, new Font ("Courier", Font.PLAIN, 10));
        Needle needle =
                new Needle(a, sp,
                          new SimpleNeedle(82.0, 1.0, 5.0, 4.2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT),
                          new Color(255, 0, 0));

        layers = new LinkedList<>();
        layers.addLast(new BackgroundLayer(Color.BLACK, frame));
        layers.addLast(new EllipticScaleTicksLayer(a, 95.0, 80.0, sp,
                                                   new TickSequenceRule(0, 10),
                                                   vp));
        layers.addLast(new EllipticScaleTicksLayer(a, 95.0, 85.0, sp,
                                                   new TickSequenceRule(5, 10),
                                                   vp));
        layers.addLast(new EllipticScaleTicksLayer(a, 95.0, 90.0, sp,
                                                   new TickSequenceRule(1, 1, 5),
                                                   vp));
        layers.addLast(new CircularArcLayer(a, 95.0, vp));
        layers.addLast(new TickLabelsOnEllipticScale(
                 a, 78.0, new TickSequenceRule(0, 10),
                vp, vsp, sp));
        layers.addLast(new GaugeLabel(vsp.copyWithSize(18), new Point2D.Double(0.0, 35.0), "km/h"));
        layers.addLast(new CachingLayer());
        layers.addLast(new NeedleLayer(needle, this.valueSource));

        instrument = new LayeredDisplayObject(layers, frame);
    }
}
