
package example001;

import drawables.AbstractLayer;
import drawables.AngularRange;
import drawables.BackgroundLayer;
import drawables.CachingLayer;
//import drawables.CircularArcLayer;
import drawables.EllipticScaleTicksLayer;
import drawables.GaugeLabel;
import drawables.LayeredDisplayObject;
import drawables.Needle;
import drawables.NeedleLayer;
import drawables.ReferenceFrame;
import drawables.ScaleProperties;
import drawables.SimpleNeedle;
import drawables.TickLabelsOnEllipticScale;
import drawables.TickSequenceRule;
import drawables.VisualProperties;
import drawables.VisualStringProperties;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.util.LinkedList;
import javax.swing.DefaultBoundedRangeModel;
import mvc.ComponentContainer;
import window.InstrumentExplorerModel;
import window.TestableInstrument;
import window.TestableInstrumentModel;

/**
 * Voltmeter, Instrument verwendet den traditionellen 90°-Bogen, die Einheit
 * wird in der linken oberen Ecke angezeigt.
 * @author Projekte
 */
public class InstrumentExample002 extends TestableInstrumentModel {

    private DefaultBoundedRangeModel valueSource;
    private LayeredDisplayObject instrument;

    public InstrumentExample002() {
        this.createValueSource();
        this.createInstrument();
    }

    @Override
    public DefaultBoundedRangeModel getSliderModel() {
        return valueSource;
    }

    @Override
    public LayeredDisplayObject getInstrument() {
        return instrument;
    }

    @Override
    public void installNewComponent(ComponentContainer<InstrumentExplorerModel> cc) {
        TestableInstrument view = new TestableInstrument(this);
        view.installInto(cc);
    }

    private void createValueSource() {
        valueSource = new DefaultBoundedRangeModel(0, 0, 0, 80*10);
    }
    private void createInstrument() {
        LinkedList<AbstractLayer> layers;
        ReferenceFrame frame = new ReferenceFrame(-100, 100, -100, 100);
        Point2D cp = new Point2D.Double(90.0d, 90.0d);
        AngularRange a = new AngularRange(cp, 180.0d, 90.0d);
        ScaleProperties sp = new ScaleProperties(0, 80, 40);
        VisualProperties vp = new VisualProperties(Color.BLACK, 1.0f);
        VisualStringProperties vsp = new VisualStringProperties(Color.BLACK, new Font ("Courier", Font.PLAIN, 12));
        VisualStringProperties labelFont = vsp.copyWithSize(34);
        Needle needle =
                new Needle(a, sp,
                          new SimpleNeedle(176.0, 1.2, 5.0, 5.2, BasicStroke.CAP_BUTT, BasicStroke.CAP_BUTT),
                          new Color(255, 0, 0));

        layers = new LinkedList<>();
        layers.addLast(new BackgroundLayer(Color.WHITE, frame));
        layers.addLast(new EllipticScaleTicksLayer(a, 184.0, 173.0, sp,
                                                   new TickSequenceRule(0, 10),
                                                   vp));
        layers.addLast(new EllipticScaleTicksLayer(a, 173.0, 165.0, sp,
                                                   new TickSequenceRule(0, 10),
                                                   new VisualProperties(Color.BLACK, 2.4f)));
        layers.addLast(new EllipticScaleTicksLayer(a, 184.0, 170.0, sp,
                                                   new TickSequenceRule(5, 10),
                                                   new VisualProperties(Color.BLACK, 1.4f)));
        layers.addLast(new EllipticScaleTicksLayer(a, 184.0, 175.0, sp,
                                                   new TickSequenceRule(1, 1, 5),
                                                   vp));

        layers.addLast(new TickLabelsOnEllipticScale(
                a, 160.0, new TickSequenceRule(0, 10),
                vp, vsp, sp));
        layers.addLast(new GaugeLabel(labelFont, new Point2D.Double(-70.0, -70.0), "V"));

        layers.addLast(new CachingLayer());
        layers.addLast(new NeedleLayer(needle, this.valueSource));

        instrument = new LayeredDisplayObject(layers, frame);
    }
}
