
package example001;

import drawables.AbstractLayer;
import drawables.BackgroundLayer;
import drawables.CachingLayer;
import drawables.FlatScaleTicksLayer;
import drawables.LayeredDisplayObject;
import drawables.NeedleOnStraightScale;
import drawables.ReferenceFrame;
import drawables.ScaleOrientation;
import drawables.ScaleProperties;
import drawables.SimpleNeedle;
import drawables.StraightNeedleLayer;
import drawables.StraightRange;
import drawables.TickLabelsOnFlatScale;
import drawables.TickSequenceRule;
import drawables.VisualProperties;
import drawables.VisualStringProperties;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.util.LinkedList;
import javax.swing.DefaultBoundedRangeModel;
import mvc.ComponentContainer;
import window.InstrumentExplorerModel;
import window.TestableMultiInstrument;
import window.TestableMultiInstrumentModel;

/**
 * Waagerechte Skala, Teilstriche über und unter den Zahlenwerten.
 * Das Instrument hat zwei Nadeln, die unterschiedliche Werte auf
 * der gleichen Skala anzeigen.
 * @author Projekte
 */
public class InstrumentExample010 extends TestableMultiInstrumentModel {

    private LayeredDisplayObject instrument;
    public InstrumentExample010() {
        super(2);
        this.createValueSources();
        this.createInstrument();
    }
    @Override
    public LayeredDisplayObject getInstrument() {
        return instrument;
    }

    @Override
    public void installNewComponent(ComponentContainer<InstrumentExplorerModel> cc) {
        TestableMultiInstrument view = new TestableMultiInstrument(this);
        view.installInto(cc);
    }

    private void createValueSources() {
        for (int i = 0; i < this.getNumberOfParameters(); i++) {
            valueHolders[i] = new DefaultBoundedRangeModel(0, 0, 0, 100);
        }
    }

    private void createInstrument() {
        LinkedList<AbstractLayer> layers;
        ReferenceFrame frame = new ReferenceFrame(-100, 100, -50, 50);
        StraightRange sr = new StraightRange(new Point2D.Double(-90.0, 0.0),
                 new Point2D.Double(1.0, 0.0),
                 ScaleOrientation.HORIZONTAL,
                 180.0);

        ScaleProperties sp = new ScaleProperties(0, 100, 100);
        VisualProperties vp1 = new VisualProperties(Color.BLACK, 1.0f);
        VisualProperties vp2 = new VisualProperties(Color.RED, 0.7f);
        VisualProperties vp3 = new VisualProperties(Color.BLACK, 0.5f);
        VisualStringProperties vsp = new VisualStringProperties(Color.BLACK, new Font ("Courier", Font.PLAIN, 10));
        NeedleOnStraightScale needle1 =
                new NeedleOnStraightScale(
                          sr,
                          sp,
                          new Color(0, 255, 0),
                          30.0,
                          new SimpleNeedle(0.0, 2.0, 29.0, 1.0,
                                  BasicStroke.CAP_BUTT,
                                  BasicStroke.CAP_BUTT));
        NeedleOnStraightScale needle2 =
                new NeedleOnStraightScale(
                          sr,
                          sp,
                          new Color(255, 0, 0),
                          -30.0,
                          new SimpleNeedle(29.0, 1.0, 0.0, 2.0,
                                  BasicStroke.CAP_BUTT,
                                  BasicStroke.CAP_BUTT));
        layers = new LinkedList<>();
        layers.addLast(new BackgroundLayer(Color.WHITE, frame));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, 25.0, 10.0,
                                                   new TickSequenceRule(0, 10),
                                                   vp1));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, 20.0, 10.0,
                                                   new TickSequenceRule(5, 10),
                                                   vp2));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, 16.0, 10.0,
                                                   new TickSequenceRule(1, 1, 5),
                                                   vp3));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, -25.0, -10.0,
                                                   new TickSequenceRule(0, 10),
                                                   vp1));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, -20.0, -10.0,
                                                   new TickSequenceRule(5, 10),
                                                   vp2));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, -16.0, -10.0,
                                                   new TickSequenceRule(1, 1, 5),
                                                   vp3));
        layers.addLast(new TickLabelsOnFlatScale(
                 sr, 0.0, new TickSequenceRule(0, 10),
                vp3, vsp, sp));
        layers.addLast(new CachingLayer());
        layers.addLast(new StraightNeedleLayer(needle1, this.valueHolders[1]));
        layers.addLast(new StraightNeedleLayer(needle2, this.valueHolders[0]));

        instrument = new LayeredDisplayObject(layers, frame);
    }
}
