
package example001;

import drawables.AbstractLayer;
import drawables.BackgroundLayer;
import drawables.CachingLayer;
import drawables.DoublePointedNeedle;
import drawables.FlatScaleTicksLayer;
import drawables.LayeredDisplayObject;
import drawables.NeedleOnStraightScale;
import drawables.ReferenceFrame;
import drawables.ScaleOrientation;
import drawables.ScaleProperties;
import drawables.StraightNeedleLayer;
import drawables.StraightRange;
import drawables.TickLabelsOnFlatScale;
import drawables.TickSequenceRule;
import drawables.VisualProperties;
import drawables.VisualStringProperties;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.geom.Point2D;
import java.util.LinkedList;
import javax.swing.DefaultBoundedRangeModel;
import mvc.ComponentContainer;
import window.InstrumentExplorerModel;
import window.TestableInstrument;
import window.TestableInstrumentModel;
/**
 *
 * @author Projekte
 */
public class InstrumentExample011  extends TestableInstrumentModel {
    private DefaultBoundedRangeModel valueSource;
    private LayeredDisplayObject instrument;

    public InstrumentExample011() {
        this.createValueSource();
        this.createInstrument();
    }
    @Override
    public DefaultBoundedRangeModel getSliderModel() {
        return valueSource;
    }

    @Override
    public LayeredDisplayObject getInstrument() {
        return instrument;
    }

    @Override
    public void installNewComponent(ComponentContainer<InstrumentExplorerModel> cc) {
        TestableInstrument view = new TestableInstrument(this);
        view.installInto(cc);
    }

    private void createValueSource() {
        valueSource = new DefaultBoundedRangeModel(0, 0, 0, 100);
    }
    private void createInstrument() {
        LinkedList<AbstractLayer> layers;
        ReferenceFrame frame = new ReferenceFrame(-50, 50, -100, 100);

        StraightRange sr = new StraightRange(new Point2D.Double(0.0, -90.0),
                 new Point2D.Double(1.0, 0.0),
                 ScaleOrientation.VERTICAL,
                 180.0);

        ScaleProperties sp = new ScaleProperties(0, 100, 100);
        VisualProperties vp1 = new VisualProperties(Color.BLACK, 1.0f);
        VisualProperties vp2 = new VisualProperties(Color.RED, 0.7f);
        VisualProperties vp3 = new VisualProperties(Color.BLACK, 0.5f);
        VisualStringProperties vsp = new VisualStringProperties(Color.BLACK, new Font ("Courier", Font.PLAIN, 10));
        NeedleOnStraightScale needle1 =
                new NeedleOnStraightScale(
                          sr,
                          sp,
                          new Color(255, 0, 0),
                          0.0,
                          new DoublePointedNeedle(25.0, 1.0, 3.5, 25.0, 1.0,
                                  BasicStroke.CAP_BUTT,
                                  BasicStroke.CAP_BUTT));

        layers = new LinkedList<>();
        layers.addLast(new BackgroundLayer(Color.WHITE, frame));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, 25.0, 10.0,
                                                   new TickSequenceRule(0, 10),
                                                   vp1));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, 20.0, 10.0,
                                                   new TickSequenceRule(5, 10),
                                                   vp2));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, 16.0, 10.0,
                                                   new TickSequenceRule(1, 1, 5),
                                                   vp3));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, -25.0, -10.0,
                                                   new TickSequenceRule(0, 10),
                                                   vp1));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, -20.0, -10.0,
                                                   new TickSequenceRule(5, 10),
                                                   vp2));
        layers.addLast(new FlatScaleTicksLayer(sr, sp, -16.0, -10.0,
                                                   new TickSequenceRule(1, 1, 5),
                                                   vp3));
        layers.addLast(new TickLabelsOnFlatScale(
                 sr, 0.0, new TickSequenceRule(0, 10),  //  add alignment here!
                vp3, vsp, sp));
        layers.addLast(new CachingLayer());
        layers.addLast(new StraightNeedleLayer(needle1, this.valueSource));

        instrument = new LayeredDisplayObject(layers, frame);
    }
}
