package mvc;

import java.util.Observable;
import java.util.Observer;
import javax.swing.JComponent;

/**
 * Eine Instanz erweitert den Container JComponent
 * und agiert als Beobachter eines ModelHolder.
 * Sobald sich der Inhalt des ModelHolders ändert, wird
 * das neue Modell aufgefordert, seine Darstellungskomponente
 * in den ComponentContainer zu installieren. Die bisher angezeigte
 * Komponente wird vorher entfernt.
 * @author B. Gaertner
 * @see IModel
 */
public class ComponentContainer
                  <ModelType  extends IModel<ModelType>>
      extends JComponent implements Observer {

    ModelHolder<ModelType> model;

    public ComponentContainer (ModelHolder<ModelType> mh)
    {
        model = mh;
        this.registerAsObserver(mh);

    }

    private void registerAsObserver(ModelHolder<ModelType> mh) {
        mh.addObserver(this);
    }

    /**
     * explicit removal of the instance from the observer list of the model
     * is done here. This is a secure approach, which is recommended because
     * it is not clear from the Java documentention whether observers are
     * weakly referenced.
     */
    @Override
    public void removeNotify() {
        super.removeNotify();
        model.deleteObserver(this);
    }

    @Override
    public void update(Observable observable, Object arg) {
        //ModelHolder<ModelType> mh = (ModelHolder<ModelType>) observable;
        ModelHolder<ModelType> mh = (ModelHolder<ModelType>) observable;
        this.removeAll();
        this.setLayout(null);
        if (mh != null) {
            if (mh.hasModel()) {
                mh.installPresenterIn(this);
            }
        }
        this.revalidate();
        this.repaint();
    }

}
