package window;

import common.SliderListener;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.util.Observable;
import java.util.Observer;
import javax.swing.DefaultBoundedRangeModel;
import javax.swing.JComponent;

/**
 * Eine Zeichenfläche, die Methoden für die Festlegung der
 * Strichbrreite bereitstellt.
 * @author Projekte
 */
public class InstrumentPaintArea extends JComponent implements Observer {

    private SimpleInstrumentModel model;

    public InstrumentPaintArea (SliderListener [] sl,
            SimpleInstrumentModel m) {
        model = m;
        if (sl != null) {
           this.setUp(sl);
        }
    }

    private void setUp (SliderListener[] sl) {
       for (int i = 0; i < sl.length; i++) {
            sl[i].addObserver(this);
       }
    }

    @Override
    public void paintComponent(Graphics arg) {
        super.paintComponent(arg);
        Dimension size = this.getSize();
        Insets insets = this.getInsets();

        Rectangle bereich = new Rectangle(insets.left, insets.top,
                size.width - (insets.left + insets.right),
                size.height - (insets.top + insets.bottom));

        Color currentColor = arg.getColor();
        arg.setColor(this.getBackground());
        arg.fillRect(bereich.x, bereich.y, bereich.width, bereich.height);
        arg.setColor(currentColor);
        Graphics2D g = (Graphics2D) arg;
        Stroke st = g.getStroke();
        this.drawInstrument(g, bereich);
        g.setStroke(st);
    }

    private void drawInstrument (Graphics2D g, Rectangle bereich){
        Graphics2D copyTool = (Graphics2D)g.create();
        model.getInstrument().renderOn(g, copyTool, null, bereich);
        copyTool.dispose();
    }


    /**
     * Wenn sich die Darstellungsgröße des Instruments ändert, müssen
     * alle cached images gelöscht werden.
     * @param x
     * @param y
     * @param width
     * @param height
     */
    @Override
    public void setBounds(int x, int y, int width, int height) {
        super.setBounds(x, y, width, height);
        model.getInstrument().invalidateCaches();
    }

    /**
     * Wenn das Darstellungspanel verworfen wird, gibt es keinen
     * Grund mehr, die cached images zu behalten - sie werden daher gelöscht.
     */
    @Override
    public void removeNotify() {
        model.stopInstrument();
        model.getInstrument().invalidateCaches();
    }
    /**
     * Mit einem Aufruf dieser Methode teilt der beobachtete SliderListener eine
     * Änderung des eingestellten Wertes mit.
     * @param source
     * @param arg
     */
    @Override
    public void  update(Observable source,
                       Object arg) {
        DefaultBoundedRangeModel valueSource = ((SliderListener) source).getCurrentValue();
        model.getInstrument().invalidateCaches(valueSource);
        this.repaint();
    }
}
