
package window;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;

import java.awt.Insets;
import java.awt.LayoutManager;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JToggleButton;
import mvc.ComponentContainer;


/**
 *
 * @author Projekte
 */
public class MainWindow extends JFrame
         implements Runnable, LayoutManager {
    /**
     * A reference to an instance that stores application data and implements application features.
     * @see ApplicationModel
     */
    private ApplicationModel model = null;
    /**
     * The entry point of the program.
     */
    @Override
    public void run() {
        model = new ApplicationModel();
        this.createGUI();
    }

     /**
     * Set up the components of the main window and make the window visible.
     */
    private void createGUI() {
        this.setTitle("Gauges");
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        this.addComponentsToPane(this.getContentPane());
        pack();
        this.setSize(500, 500);
        this.setLocationRelativeTo(null); // center on screen
        this.setVisible(true);
    }

    /**
     * The method creates the visual elements that comprise the window.
     * @param container
     */
    private void addComponentsToPane (Container container) {
        container.setLayout(this);
        ButtonGroup buttonGroup = new ButtonGroup();
        JToggleButton bttn1 = new JToggleButton("Button 1");
        JToggleButton bttn2 = new JToggleButton("Button 2");
        JToggleButton bttn3 = new JToggleButton("Button 3");
        JToggleButton bttn4 = new JToggleButton("Button 4");
        JToggleButton bttn5 = new JToggleButton("Button 5");
        JToggleButton bttn6 = new JToggleButton("Button 6");
        JToggleButton bttn7 = new JToggleButton("Button 7");
        JToggleButton bttn8 = new JToggleButton("Button 8");

        ComponentContainer<InstrumentExplorerModel> aspectView =
                new ComponentContainer<>(model.getExplorerModel());

        bttn1.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_01));
        bttn2.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_02));
        bttn3.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_03));
        bttn4.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_04));
        bttn5.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_05));
        bttn6.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_06));
        bttn7.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_07));
        bttn8.addActionListener(new ButtonChangeAdapter(model, Aspect.ITEM_08));

        buttonGroup.add(bttn1);
        buttonGroup.add(bttn2);
        buttonGroup.add(bttn3);
        buttonGroup.add(bttn4);
        buttonGroup.add(bttn5);
        buttonGroup.add(bttn6);
        buttonGroup.add(bttn7);
        buttonGroup.add(bttn8);

        container.add(bttn1);
        container.add(bttn2);
        container.add(bttn3);
        container.add(bttn4);
        container.add(bttn5);
        container.add(bttn6);
        container.add(bttn7);
        container.add(bttn8);
        container.add(aspectView);
    }

    @Override
    public void addLayoutComponent(String name, Component comp) {
        //  no action needed
    }

    @Override
    public void removeLayoutComponent(Component comp) {
        //  no action needed
    }

    @Override
    public Dimension preferredLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        //Always add the container's insets!
        Insets insets = parent.getInsets();
        dim.width = 100
                    + insets.left + insets.right;
        dim.height = 100
                     + insets.top + insets.bottom;

        return dim;
    }

    @Override
    public Dimension minimumLayoutSize(Container parent) {
        Dimension dim = new Dimension(0, 0);

        //Always add the container's insets!
        Insets insets = parent.getInsets();
        dim.width = 60 + insets.left + insets.right;
        dim.height = 26 + insets.top + insets.bottom;

        return dim;
    }

    @Override
    public void layoutContainer(Container parent) {
        Insets insets = parent.getInsets();
        final int maxWidth = parent.getWidth() - (insets.left + insets.right);
        final int maxHeight = parent.getHeight() - (insets.top + insets.bottom);
        final int x0 = insets.left;
        final int y0 = insets.top;
        final int buttonHeight = 26;
        final int buttonsPerRow = 4;
        int x;
        int w;

        // now, compute x origin and width for all buttons
        for (int i = 0; i < buttonsPerRow; i++) {
            Component c = parent.getComponent(i);
            w = maxWidth * (i + 1) / buttonsPerRow - maxWidth * i/buttonsPerRow;
            x = maxWidth * i / buttonsPerRow;
            c.setBounds(x0 + x, y0, w, buttonHeight);
            c = parent.getComponent(i + buttonsPerRow);
            c.setBounds(x0 + x, y0 + buttonHeight, w, buttonHeight);
        }
        //  compute the placement bounds of the instrument display component
        Component c = parent.getComponent(buttonsPerRow*2);
        c.setBounds(x0, y0 + buttonHeight*2, maxWidth, maxHeight - buttonHeight*2);
    }

}
